use kdl::KdlDocument;
use miette::{miette, Result};
use std::path::PathBuf;

#[derive(Debug, Clone)]
pub struct Config {
    pub endpoint: String,
    pub pairs_file: PathBuf,
    pub concurrency: usize,
}

impl Config {
    pub fn parse_config(config_file: PathBuf) -> Result<Config> {
        let config_str = std::fs::read_to_string(config_file).map_err(|e| miette!(e))?;

        let doc = config_str.parse::<KdlDocument>()?;

        let endpoint = doc
            .get("endpoint")
            .ok_or(miette!("Missing 'endpoint'"))?
            .get(0)
            .ok_or(miette!("'endpoint' has no value"))?
            .as_string()
            .ok_or(miette!("'endpoint' value must be a string"))?
            .to_string();

        let pairs_file = PathBuf::from(
            doc.get("pairs_file")
                .ok_or(miette!("Missing 'pairs_file'"))?
                .get(0)
                .ok_or(miette!("'pairs_file' has no value"))?
                .as_string()
                .ok_or(miette!("'pairs_file' value must be a string"))?
                .to_string(),
        );

        let concurrency =
            doc.get("concurrency")
                .ok_or(miette!("Missing 'concurrency'"))?
                .get(0)
                .ok_or(miette!("'concurrency' has no value"))?
                .as_integer()
                .ok_or(miette!("'concurrency' value must be an integer"))? as usize;

        Ok(Self {
            endpoint,
            pairs_file,
            concurrency,
        })
    }
}
