mod pairs;

use alloy::primitives::Address;
use futures_util::Stream;
use log::debug;
use std::{
    pin::Pin,
    task::{Context, Poll},
};

#[derive(Debug)]
pub enum Action {
    ProcessPair(Address),
}

pub struct PriorityQueue(pub Vec<Action>);

impl PriorityQueue {
    pub fn new() -> Self {
        PriorityQueue(Vec::new())
    }

    pub fn push(&mut self, action: Action) {
        debug!("Adding action {:?} to the priority queue", action);
        self.0.push(action);
    }
}

impl Stream for PriorityQueue {
    type Item = Action;

    fn poll_next(mut self: Pin<&mut Self>, _cx: &mut Context<'_>) -> Poll<Option<Self::Item>> {
        match self.0.pop() {
            None => Poll::Ready(None),
            Some(action) => {
                debug!("Consuming action {:?} to the priority queue", action);

                match action {
                    Action::ProcessPair(pair) => Poll::Ready(Some(Action::ProcessPair(pair))),
                }
            }
        }
    }
}
